package cn.teammodel.controller.admin.service.impl;

import cn.teammodel.common.ErrorCode;
import cn.teammodel.common.PK;
import cn.teammodel.config.exception.ServiceException;
import cn.teammodel.controller.admin.service.TeacherService;
import cn.teammodel.model.dto.admin.teacher.GpTeacherDto;
import cn.teammodel.model.dto.admin.teacher.TeacherDto;
import cn.teammodel.model.entity.common.GroupList;
import cn.teammodel.model.entity.school.GpTeacher;
import cn.teammodel.model.entity.school.LessonRecord;
import cn.teammodel.model.entity.school.School;
import cn.teammodel.model.entity.school.SchoolTeacher;
import cn.teammodel.model.vo.admin.GpTeacherVo;
import cn.teammodel.model.vo.admin.TeacherGradeVo;
import cn.teammodel.repository.LessonRecordRepository;
import cn.teammodel.repository.SchoolGroupListRepository;
import cn.teammodel.repository.SchoolRepository;
import cn.teammodel.repository.SchoolTeacherRepository;
import cn.teammodel.utils.GroupUtil;
import cn.teammodel.utils.SchoolDateUtil;
import com.azure.spring.data.cosmos.core.query.CosmosPageRequest;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.data.domain.Slice;
import org.springframework.data.domain.SliceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.time.*;
import java.util.*;
import java.util.stream.Collectors;

import static cn.teammodel.utils.SchoolDateUtil.calculateWeekNum;

@Service
public class TeacherServiceImpl implements TeacherService {
    @Resource
    private SchoolTeacherRepository schoolTeacherRepository;
        @Resource
    private SchoolGroupListRepository schoolGroupListRepository;
    @Resource
    private SchoolRepository  schoolRepository;
    @Resource
    private LessonRecordRepository lessonRecordRepository;
    @Autowired
    private Environment env;
    @Override
    public List<Map<String, Integer>> getTeacherList(TeacherDto teacherDto) {
        List<Map<String, Integer>> result = new ArrayList<>();
        Map<String, Integer> teacherCountMap = new HashMap<>();
        Map<String, Integer> groupCountMap = new HashMap<>();
       List<GroupList> groupLists = schoolGroupListRepository.findAllByCode("GroupList-" + teacherDto.getCode());
       List<SchoolTeacher> schoolTeachers = schoolTeacherRepository.findAllByCode("Teacher-" + teacherDto.getCode());
       teacherCountMap.put("教师总人数", schoolTeachers.size());
       List<School> school = schoolRepository.findBaseById(teacherDto.getCode());
       List<School.Subject> subjects = new ArrayList<>();
       List<School.Period> periods = new ArrayList<>();
       for (School sc : school) {
           if (sc.getPeriod() != null) {
               // 遍历每个 Period 对象
               for (School.Period period : sc.getPeriod()) {
                   periods.add(period);
                   // 检查 period 是否有 subjects 列表
                   if (period.getSubjects() != null) {
                       // 将 period 中的 subjects 添加到总的 subjects 列表中
                       subjects.addAll(period.getSubjects());
                   }
               }
           }

       }

        Map<String, Integer> updatedSubjectCountMap = getStringIntegerMap(subjects, schoolTeachers);

        // 创建一个映射，用于存储每个职位的教师数量
        Map<String, Integer> jobCountMap = new HashMap<>();

        // 遍历所有教师
        for (SchoolTeacher teacher : schoolTeachers) {
            // 获取教师的职位
            String job = teacher.getJob();
            if (job != null) {
                // 更新每个职位的教师数量
                jobCountMap.put(job, jobCountMap.getOrDefault(job, 0) + 1);
            }
        }
        jobCountMap.entrySet().removeIf(entry -> entry.getKey().isEmpty());
        Map<String, Integer> periodCountMap = getPeriodCountMap(periods,schoolTeachers);
        for (GroupList groupList : groupLists) {
            groupCountMap.put(groupList.getName(), groupList.getMembers().size());
        }
        periodCountMap.entrySet().removeIf(entry -> entry.getKey().isEmpty());
        updatedSubjectCountMap.entrySet().removeIf(entry -> entry.getKey().isEmpty());
        result.add(updatedSubjectCountMap);
        result.add(jobCountMap);
        result.add(periodCountMap);
        result.add(teacherCountMap);
        result.add(groupCountMap);
        return result;
    }

    @Override
    public Map<String, Object> getTeacherDetail(TeacherDto teacherDto, HttpServletRequest request) {

        Map<String, Object> teachers;
        //获取全学校教师名单详情
        String url = env.getProperty("ies.server-url-get-teacher-all");
        try {
            teachers =  GroupUtil.getGroupId(teacherDto,new GroupUtil(env), request,url);
            List<GpTeacherVo> teachersList = new ArrayList<>();
            for (Map.Entry<String, Object> entry : teachers.entrySet()) {
                // 假设 entry.getValue() 返回的是 List<Map<String, Object>>
                List<Map<String, Object>> mapList = (List<Map<String, Object>>) entry.getValue();
                for (Map<String, Object> map : mapList) {
                    // 将 Map 转换为 GpTeacherVo 对象
                    GpTeacherVo teacher = convertToGpTeacherVo(map);
                    teachersList.add(teacher);
                }
            }
            List<String> phones = new ArrayList<>();
            // 进一步处理 teachersList
            for (GpTeacherVo teacher : teachersList) {
                phones.add(teacher.getPhone());
            }
            GpTeacherDto gpTeacherDto = new GpTeacherDto();
            gpTeacherDto.setMobiles(phones);
            Map<String, Object> tBase = getTeacherBase(gpTeacherDto, request);
            List<GpTeacher> gpTeachers = new ArrayList<>();
            for (Map.Entry<String, Object> entry : tBase.entrySet()) {
                if (entry.getKey().equals("teachers")) {
                    List<Map<String, Object>> dataList = (List<Map<String, Object>>) entry.getValue();
                    for (Map<String, Object> dataMap : dataList) {
                        GpTeacher gpTeacher = convertToGpTeacher(dataMap);
                        gpTeachers.add(gpTeacher);
                    }
                }
            }
            Map<String, Long> genderCount = gpTeachers.stream()
                    .collect(Collectors.groupingBy(gpTeacher -> gpTeacher.getBasic().getGender(), Collectors.counting()));
            Map<String, Long> degreeCount = gpTeachers.stream()
                    .filter(gpTeacher -> gpTeacher.getDegree() != null)
                    .collect(Collectors.groupingBy(GpTeacher::getDegree, Collectors.counting()));
            //Map<String, Integer> practiceCount = getStringIntegerMap(teacherDto, gpTeachers);

            if(!genderCount.containsKey("F")) {
                genderCount.put("F",0L);
            }else if (!genderCount.containsKey("S")) {
                genderCount.put("S",0L);
            }else if (!genderCount.containsKey("M")){
                genderCount.put("M",0L);
            }
           /* Map<String, Object> genderMap = new HashMap<>();
            Map<String, Object> practiceMap = new HashMap<>();
            for (GpTeacher gpTeacher : gpTeachers) {
                genderMap.put(gpTeacher.basic.name, gpTeacher.basic.gender);
                practiceMap.put(gpTeacher.basic.name, gpTeacher.practiceTime);
            }*/

            for (GpTeacherVo teacher : teachersList) {
                for(GpTeacher gpTeacher : gpTeachers) {
                    if(StringUtils.isNotEmpty(teacher.phone) && teacher.phone.equalsIgnoreCase(gpTeacher.mobile)) {
                        if(gpTeacher.basic.gender != null) {
                            teacher.setGender(gpTeacher.basic.gender);
                        }else {
                            teacher.setGender("");
                        }
                        if (gpTeacher.getPracticeTime() != null && gpTeacher.getPracticeTime() > 0) {
                            teacher.setPracticeTime(gpTeacher.getPracticeTime());
                        }else {
                            teacher.setPracticeTime(0L);
                        }
                        if(gpTeacher.degree != null) {
                            teacher.setDegree(gpTeacher.degree);
                        }else {
                            teacher.setDegree("");
                        }
                        break;
                    }
                }
            }
            //teachers.put("practice", practiceCount);
            //teachers.put("practiceMap", practiceMap);
            //teachers.put("genderMap", genderMap);
            teachers.put("gender", genderCount);
            teachers.put("degree", degreeCount);
            teachers.put("info", teachersList);
            teachers.remove("teachers");

        }catch (Exception e) {
            throw new ServiceException(ErrorCode.SYSTEM_ERROR.getCode(), "数据转换错误");

        }
        return teachers;
    }

    private static @NotNull Map<String, Integer> getStringIntegerMap(TeacherDto teacherDto, List<GpTeacher> gpTeachers) {
        Map<String,Integer> practiceCount = new HashMap<>();
        for (GpTeacher gpTeacher : gpTeachers) {
            if(gpTeacher.practiceTime > 0) {
                // 将时间戳转换为 LocalDateTime
                LocalDateTime dateTime = LocalDateTime.ofInstant(
                        Instant.ofEpochMilli(gpTeacher.practiceTime),
                        ZoneId.systemDefault()
                );

                // 获取年份
                int year = dateTime.getYear();
                practiceCount.put(gpTeacher.basic.name, teacherDto.year - year);
            }
        }
        return practiceCount;
    }

    private GpTeacherVo convertToGpTeacherVo(Map<String, Object> map) {
        GpTeacherVo teacher = new GpTeacherVo();
        // 假设 GpTeacherVo 有以下属性：name, age, subject
        teacher.setName((String) map.get("name"));
        teacher.setPhone((String) map.get("phone"));
        teacher.setId((String) map.get("id"));
        teacher.setEmail((String) map.get("email"));
        teacher.setPicture((String) map.get("picture"));
        teacher.setIname((String) map.get("iname"));
        teacher.setJob((String) map.get("job"));
        teacher.setRoles((List<String>) map.get("roles"));
        //teacher.setGroups((List<GpTeacherVo.IdNameCode>) map.get("groups"));
        List<Map<String, Object>> groupList = (List<Map<String, Object>>) map.get("groups");
        if (groupList != null) {

            List<GpTeacherVo.IdNameCode> groups = new ArrayList<>();
            for (Map<String, Object> groupMap : groupList) {
                GpTeacherVo.IdNameCode group = new GpTeacherVo.IdNameCode();
                group.setId((String) groupMap.get("id"));
                group.setName((String) groupMap.get("name"));
                group.setCode((String) groupMap.get("code"));
                group.setPicture((String) groupMap.get("picture"));
                group.setNickname((String) groupMap.get("nickname"));
                groups.add(group);
            }
            teacher.setGroups(groups);
        }
        //teacher.setDegree((String) map.get("degree"));
        //teacher.setPracticeTime((String) map.get("practiceTime"));
        //teacher.setGender((String) map.get("gender"));
        teacher.setNote((String) map.get("note"));
        teacher.setSubjectIds((List<String>) map.get("subjectIds"));
        teacher.setSubjectNames((List<String>) map.get("subjectNames"));
        // 根据实际属性进行转换
        return teacher;
    }

    private TeacherGradeVo convertToTeacherGradeVo(Map<String, Object> map) {
        TeacherGradeVo teacher = new TeacherGradeVo();
        teacher.setName((String) map.get("name"));
        teacher.setId((String) map.get("tmdId"));
        // 处理 grade 字段
        List<Map<String, Object>> gradeList = (List<Map<String, Object>>) map.get("grade");
        if (gradeList != null) {
            List<TeacherGradeVo.IdAndName> grades = new ArrayList<>();
            for (Map<String, Object> gradeMap : gradeList) {
                TeacherGradeVo.IdAndName grade = new TeacherGradeVo.IdAndName();
                grade.setGrade((Integer) gradeMap.get("grade"));
                grade.setGradeName((String) gradeMap.get("gradeName"));
                grades.add(grade);
            }
            teacher.setGrade(grades);
        }
        // 根据实际属性进行转换
        return teacher;
    }
    private GpTeacher convertToGpTeacher(Map<String, Object> map) {
        GpTeacher teacher = new GpTeacher();
        // 假设 GpTeacherVo 有以下属性：name, age, subject
        Object describeObj = map.get("describe");
        if (describeObj instanceof String) {
            teacher.setDescribe((String) describeObj);
        } else {
            teacher.setDescribe(null); // 或者设置一个默认值
        }
        Object IdObj = map.get("id");
        if (describeObj instanceof String) {
            teacher.setId((String) IdObj);
        } else {
            teacher.setId(null); // 或者设置一个默认值
        }
        Object depictObj = map.get("depict");
        if (depictObj instanceof String) {
            teacher.setDepict((String) depictObj);
        } else {
            teacher.setDepict(null); // 或者设置一个默认值
        }
        teacher.setPeriods((ArrayList<GpTeacher.GnericBase>) map.get("periods"));
        teacher.setSubjects((ArrayList<GpTeacher.GnericBase>) map.get("subjects"));
        teacher.setGrades((ArrayList<GpTeacher.GnericBase>) map.get("grades"));
        Object practiceTimeObj = map.get("practiceTime");
        if (practiceTimeObj instanceof Long) {
            teacher.setPracticeTime((Long) practiceTimeObj);
        } else {
            teacher.setPracticeTime(0L); // 或者设置一个默认值
        }
        Object titleObj = map.get("title");
        if (titleObj instanceof String) {
            teacher.setTitle((String) titleObj);
        } else {
            teacher.setTitle(null); // 或者设置一个默认值
        }
        Object mobileObj = map.get("mobile");
        if (mobileObj instanceof String) {
            teacher.setMobile((String) mobileObj);
        } else {
            teacher.setMobile(null); // 或者设置一个默认值
        }
        Object degreeObj = map.get("degree");
        if (degreeObj instanceof String) {
            teacher.setDegree((String) degreeObj);
        } else {
            teacher.setDegree(null); // 或者设置一个默认值
        }
        Map<String, Object> basicMap = (Map<String, Object>) map.get("basic");
        //teacher.setBasic(new GpTeacher.GPBasicInfo(basicMap.get("name").toString(),basicMap.get("fullName").toString(),basicMap.get("picture").toString(),basicMap.get("region").toString(),basicMap.get("province").toString(),basicMap.get("city").toString(),basicMap.get("dist").toString(),basicMap.get("gender").toString(),basicMap.get("age").toString(),basicMap.get("address").toString()));
        GpTeacher.GPBasicInfo basic = new GpTeacher.GPBasicInfo();
        Object nameObj = basicMap.get("name");
        if (nameObj instanceof String) {
            basic.setName((String) nameObj);
        } else {
            basic.setName(null); // 或者设置一个默认值
        }
        Object fullNameObj = basicMap.get("fullName");
        if (fullNameObj instanceof String) {
            basic.setFullName((String) fullNameObj);
        } else {
            basic.setFullName(null); // 或者设置一个默认值
        }
        Object distObj = basicMap.get("dist");
        if (distObj instanceof String) {
            basic.setDist((String) distObj);
        } else {
            basic.setDist(null); // 或者设置一个默认值
        }
        Object genderObj = basicMap.get("gender");
        if (genderObj instanceof String) {
            basic.setGender((String) genderObj);
        } else {
            basic.setGender(null); // 或者设置一个默认值
        }
        Object ageObj = basicMap.get("age");
        if (ageObj instanceof Integer) {
            basic.setAge((Integer) ageObj);
        } else {
            basic.setAge(0); // 或者设置一个默认值
        }
        teacher.basic = basic;


        // 根据实际属性进行转换
        return teacher;
    }

    @Override
    public Map<Long, Integer> getTeacherByRecord(TeacherDto teacherDto) {
        final int SLICE_SIZE = 100;
        // 获取学期起止时间
        List<School.Semester> semesters = schoolRepository.findSemestersById(teacherDto.getCode(), teacherDto.getPeriodId());
        SchoolDateUtil.semesterModel semesterModel = SchoolDateUtil.getSemesterByNow(semesters, LocalDate.now());
        LocalDateTime startDatetime;
        LocalDateTime endDatetime;
        if(teacherDto.getStartTime() != null) {
            startDatetime = LocalDateTime.ofInstant(Instant.ofEpochMilli(teacherDto.getStartTime()), ZoneId.systemDefault());
            endDatetime = LocalDateTime.ofInstant(Instant.ofEpochMilli(teacherDto.getEndTime()), ZoneId.systemDefault());
        }else {
            startDatetime = semesterModel.getStartDatetime();
            endDatetime = semesterModel.getEndDatetime();
        }
        if (startDatetime == null || endDatetime == null) throw new ServiceException(ErrorCode.PARAMS_ERROR);
        long totalWeek = calculateWeekNum(startDatetime, endDatetime, null);

        // slice 分段读取
        CosmosPageRequest pageRequest = new CosmosPageRequest(0, SLICE_SIZE, null);
        Slice<LessonRecord> slice;
        Map<Long, Integer> countByWeek = SchoolDateUtil.createEmptyWeekMap(totalWeek);

        do {
            if(StringUtils.isNotEmpty(teacherDto.getGrade())) {
                slice = lessonRecordRepository.findAllByGrade(String.format(PK.PK_LESSON_RECORD, teacherDto.getCode()), teacherDto.getGrade(),teacherDto.getSubjectId(),pageRequest);
            } else if(StringUtils.isNotEmpty(teacherDto.getTmdId())) {
                slice = lessonRecordRepository.findAllByAcademicYearId(String.format(PK.PK_LESSON_RECORD, teacherDto.getCode()), teacherDto.getTmdId(),teacherDto.getSubjectId(),pageRequest);
            } else {
                slice = lessonRecordRepository.findAll(String.format(PK.PK_LESSON_RECORD, teacherDto.getCode()), teacherDto.getSubjectId(), pageRequest);
            }
            List<LessonRecord> content = slice.getContent();
            if (ObjectUtils.isEmpty(content)) {
                break;
            }

            // 分批次计算
            for (LessonRecord item : content) {
                // 处理每周的课程数
                long weekNum = calculateWeekNum(startDatetime, endDatetime, item.getStartTime());
                countByWeek.put(weekNum, countByWeek.getOrDefault(weekNum, 0) + 1);
            }

            if (slice.hasNext()) {
                pageRequest = (CosmosPageRequest) slice.nextPageable();
            }
        } while (slice.hasNext());
        countByWeek.entrySet().removeIf(entry -> entry.getKey() == -1);
        return countByWeek;
    }

    @Override
    public Map<String, Object> getTeacherLearningCategory(TeacherDto teacherDto) {

        Map<String, Object> LearningCategory = new HashMap<>(Collections.emptyMap());
        try {
            // 获取学期起止时间
            List<School.Semester> semesters = schoolRepository.findSemestersById(teacherDto.getCode(), teacherDto.getPeriodId());
            SchoolDateUtil.semesterModel semesterModel = SchoolDateUtil.getSemesterByNow(semesters, LocalDate.now());
            LocalDateTime startDatetime;
            LocalDateTime endDatetime;
            if(teacherDto.getStartTime() != null) {
                startDatetime = LocalDateTime.ofInstant(Instant.ofEpochMilli(teacherDto.getStartTime()), ZoneId.systemDefault());
                endDatetime = LocalDateTime.ofInstant(Instant.ofEpochMilli(teacherDto.getEndTime()), ZoneId.systemDefault());
            }else {
                startDatetime = semesterModel.getStartDatetime();
                endDatetime = semesterModel.getEndDatetime();
            }
            if (startDatetime == null || endDatetime == null) throw new ServiceException(ErrorCode.PARAMS_ERROR);

            // 将 LocalDateTime 转换为 Long 类型的时间戳
            long startTimestamp = startDatetime.toInstant(ZoneOffset.UTC).toEpochMilli();
            long endTimestamp = endDatetime.toInstant(ZoneOffset.UTC).toEpochMilli();

            List<LessonRecord> lessonRecords = lessonRecordRepository.getLessonsByConditions(String.format(PK.PK_LESSON_RECORD, teacherDto.getCode()), startTimestamp, endTimestamp, teacherDto.getSubjectId(),teacherDto.getTmdId());
            int[] counts = {0, 0, 0, 0, 0};
            String[] keys = {"cooperation", "internet", "task", "exam", "diffential"};

            for (LessonRecord record : lessonRecords) {
                if (record.getLearningCategory() != null) {
                    counts[0] += record.getLearningCategory().getCooperation();
                    counts[1] += record.getLearningCategory().getInteraction();
                    counts[2] += record.getLearningCategory().getTask();
                    counts[3] += record.getLearningCategory().getExam();
                    counts[4] += record.getLearningCategory().getDiffential();
                }
            }

            for (int i = 0; i < keys.length; i++) {
                LearningCategory.put(keys[i], counts[i]);
            }
        }catch (Exception e) {
            throw new ServiceException(ErrorCode.SYSTEM_ERROR.getCode(), "数据转换错误");

        }
        return LearningCategory;
    }

    @Override
    public Map<String, Object> getTeacherGradeCount(TeacherDto teacherDto, HttpServletRequest request) {

        Map<String, Object> gradeMap;
        String url = env.getProperty("ies.server-url-get-teacher-grade-count");
        try {
            gradeMap =  GroupUtil.getGroupId(teacherDto,new GroupUtil(env), request,url);
        }catch (Exception e) {
            throw new ServiceException(ErrorCode.SYSTEM_ERROR.getCode(), "数据转换错误");

        }
        return gradeMap;
    }

    @Override
    public List<Map<String,Map<String, Long>>> getDistributionOfTeachers(TeacherDto teacherDto, HttpServletRequest request) {

        List<Map<String,Map<String, Long>>> distributionList = new ArrayList<>();
        Map<String,Map<String, Long>>  gradesMap = new HashMap<>();
        Map<String,Map<String, Long>> degreesMap = new HashMap<>();
        Map<String,Map<String, Long>> subjectsMap= new HashMap<>();
        Map<String,Map<String, Long>> groupsMap = new HashMap<>();
        Map<String, Object> teachers;
        //获取全学校教师名单详情
        String url = env.getProperty("ies.server-url-get-teacher-all");
        try {
            teachers =  GroupUtil.getGroupId(teacherDto,new GroupUtil(env), request,url);
            List<GpTeacherVo> teachersList = new ArrayList<>();
            for (Map.Entry<String, Object> entry : teachers.entrySet()) {
                // 假设 entry.getValue() 返回的是 List<Map<String, Object>>
                List<Map<String, Object>> mapList = (List<Map<String, Object>>) entry.getValue();
                for (Map<String, Object> map : mapList) {
                    // 将 Map 转换为 GpTeacherVo 对象
                    GpTeacherVo teacher = convertToGpTeacherVo(map);
                    teachersList.add(teacher);
                }
            }
            List<String> phones = new ArrayList<>();
            // 进一步处理 teachersList
            for (GpTeacherVo teacher : teachersList) {
                phones.add(teacher.getPhone());
            }
            GpTeacherDto gpTeacherDto = new GpTeacherDto();
            gpTeacherDto.setMobiles(phones);
            Map<String, Object> tBase = getTeacherBase(gpTeacherDto, request);
            List<GpTeacher> gpTeachers = new ArrayList<>();
            for (Map.Entry<String, Object> entry : tBase.entrySet()) {
                if (entry.getKey().equals("teachers")) {
                    List<Map<String, Object>> dataList = (List<Map<String, Object>>) entry.getValue();
                    for (Map<String, Object> dataMap : dataList) {
                        GpTeacher gpTeacher = convertToGpTeacher(dataMap);
                        gpTeachers.add(gpTeacher);
                    }
                }
            }
            //把对应年级的信息进行映射
            List<TeacherGradeVo> gradeList = new ArrayList<>();
            Map<String,Object> gradeInfo =  getTeacherGradeCount(teacherDto, request);
            for (Map.Entry<String, Object> entry : gradeInfo.entrySet()) {
                if (entry.getKey().equals("teacher")) {
                    List<Map<String, Object>> dataList = (List<Map<String, Object>>) entry.getValue();
                    for (Map<String, Object> dataMap : dataList) {
                        TeacherGradeVo teacherGradeVo = convertToTeacherGradeVo(dataMap);
                        gradeList.add(teacherGradeVo);
                    }
                }
            }
            for (GpTeacherVo teacher : teachersList) {
                for(GpTeacher gpTeacher : gpTeachers) {
                    if(StringUtils.isNotEmpty(teacher.phone) && teacher.phone.equalsIgnoreCase(gpTeacher.mobile)) {
                        if(gpTeacher.basic.gender != null) {
                            teacher.setGender(gpTeacher.basic.gender);
                        }else {
                            teacher.setGender("");
                        }
                        if (gpTeacher.getPracticeTime() != null && gpTeacher.getPracticeTime() > 0) {
                            teacher.setPracticeTime(gpTeacher.getPracticeTime());
                        }else {
                            teacher.setPracticeTime(0L);
                        }
                        if(gpTeacher.degree != null) {
                            teacher.setDegree(gpTeacher.degree);
                        }else {
                            teacher.setDegree("");
                        }
                        break;
                    }
                }
                for (TeacherGradeVo grade : gradeList) {
                    if(teacher.getId().equals(grade.getId())) {
                        List<TeacherGradeVo.IdAndName> grades = grade.getGrade();
                        for (TeacherGradeVo.IdAndName gg : grades) {
                            String id = String.valueOf(teacherDto.year - gg.getGrade());
                            if (teacher.grade == null) {
                                teacher.setGrade(new ArrayList<>());
                            }
                            teacher.grade.add(id);

                        }
                    }
                }
            }

            Map<String, Long> gradeCount = new HashMap<>();
            Map<String, Long> degreeCount = new HashMap<>();
            Map<String, Long> subjectCount = new HashMap<>();
            Map<String, Long> GroupCount = new HashMap<>();
            if(StringUtils.isNotEmpty(teacherDto.getGrade()) && StringUtils.isNotEmpty(teacherDto.getSubjectId())) {
                for (GpTeacherVo teacher : teachersList) {
                    if(teacher.getGrade() != null &&teacher.getSubjectIds() != null &&teacher.getGrade().contains(teacherDto.getGrade()) && teacher.getSubjectIds().contains(teacherDto.getSubjectId())) {
                        {
                            Long count = gradeCount.getOrDefault(teacherDto.getGrade(), 0L);
                            gradeCount.put(teacherDto.getGrade(), count + 1);
                        }
                        if(StringUtils.isNotEmpty(teacher.getDegree())) {
                            {
                                Long count = degreeCount.getOrDefault(teacher.getDegree(), 0L);
                                degreeCount.put(teacher.getDegree(), count + 1);
                            }
                        }
                        {
                            int index = teacher.subjectIds.indexOf(teacherDto.getSubjectId());
                            String name = "";
                            if (index >= 0) {
                                name = teacher.subjectNames.get(index);
                            }
                            Long count = subjectCount.getOrDefault(name, 0L);
                            subjectCount.put(name, count + 1);
                        }
                        if(teacher.getGroups() != null) {
                            for (GpTeacherVo.IdNameCode group : teacher.getGroups()) {
                                Long count = GroupCount.getOrDefault(group.getName(), 0L);
                                GroupCount.put(group.getName(), count + 1);
                            }
                        }
                    }
                }
            }else if(StringUtils.isEmpty(teacherDto.getGrade()) && StringUtils.isNotEmpty(teacherDto.getSubjectId())) {
                for (GpTeacherVo teacher : teachersList) {
                    if(teacher.getSubjectIds() != null && teacher.getSubjectIds().contains(teacherDto.getSubjectId())) {
                        if(teacher.getGrade() != null) {
                            for (String grade : teacher.getGrade()) {
                                Long count = gradeCount.getOrDefault(grade, 0L);
                                gradeCount.put(grade, count + 1);
                            }
                        }
                        if(StringUtils.isNotEmpty(teacher.getDegree())) {
                            {
                                Long count = degreeCount.getOrDefault(teacher.getDegree(), 0L);
                                degreeCount.put(teacher.getDegree(), count + 1);
                            }
                        }
                        {
                            int index = teacher.subjectIds.indexOf(teacherDto.getSubjectId());
                            String name = "";
                            if (index >= 0) {
                                name = teacher.subjectNames.get(index);
                            }
                            Long count = subjectCount.getOrDefault(name, 0L);
                            subjectCount.put(name, count + 1);
                        }
                        if(teacher.getGroups() != null) {
                            for (GpTeacherVo.IdNameCode group : teacher.getGroups()) {
                                Long count = GroupCount.getOrDefault(group.getName(), 0L);
                                GroupCount.put(group.getName(), count + 1);
                            }
                        }
                    }
                }
                }else if(StringUtils.isNotEmpty(teacherDto.getGrade()) && StringUtils.isEmpty(teacherDto.getSubjectId())) {

                for (GpTeacherVo teacher : teachersList) {
                    if(teacher.getGrade() != null && teacher.getGrade().contains(teacherDto.getGrade())) {
                        {
                            Long count = gradeCount.getOrDefault(teacherDto.getGrade(), 0L);
                            gradeCount.put(teacherDto.getGrade(), count + 1);
                        }
                        if(StringUtils.isNotEmpty(teacher.getDegree())) {
                            {
                                Long count = degreeCount.getOrDefault(teacher.getDegree(), 0L);
                                degreeCount.put(teacher.getDegree(), count + 1);
                            }
                        }
                        if(teacher.getSubjectIds()!= null) {
                            for (String subjectId : teacher.getSubjectIds()) {
                                int index = teacher.subjectIds.indexOf(subjectId);
                                String name = "";
                                if (index >= 0) {
                                    name = teacher.subjectNames.get(index);
                                }
                                Long count = subjectCount.getOrDefault(name, 0L);
                                subjectCount.put(name, count + 1);
                            }
                        }
                        if(teacher.getGroups() != null) {
                            for (GpTeacherVo.IdNameCode group : teacher.getGroups()) {
                                Long count = GroupCount.getOrDefault(group.getName(), 0L);
                                GroupCount.put(group.getName(), count + 1);
                            }
                        }
                    }
                }

            }else {
                for (GpTeacherVo teacher : teachersList) {
                    if(teacher.getGrade() != null) {
                        for (String grade : teacher.getGrade()) {
                            Long count = gradeCount.getOrDefault(grade, 0L);
                            gradeCount.put(grade, count + 1);
                        }
                    }
                        if(StringUtils.isNotEmpty(teacher.getDegree())) {
                            {
                                Long count = degreeCount.getOrDefault(teacher.getDegree(), 0L);
                                degreeCount.put(teacher.getDegree(), count + 1);
                            }
                        }
                        if(teacher.getSubjectIds()!= null) {
                            for (String subjectId : teacher.getSubjectIds()) {
                                int index = teacher.subjectIds.indexOf(subjectId);
                                String name = "";
                                if (index >= 0) {
                                    name = teacher.subjectNames.get(index);
                                }
                                Long count = subjectCount.getOrDefault(name, 0L);
                                subjectCount.put(name, count + 1);
                            }
                        }
                        if(teacher.getGroups() != null) {
                            for (GpTeacherVo.IdNameCode group : teacher.getGroups()) {
                                Long count = GroupCount.getOrDefault(group.getName(), 0L);
                                GroupCount.put(group.getName(), count + 1);
                            }
                        }

                }
            }
            gradeCount.keySet().removeIf(key -> key.startsWith("20"));
            gradesMap.put("grade", gradeCount);
            degreesMap.put("degree", degreeCount);
            subjectsMap.put("subject", subjectCount);
            groupsMap.put("group", GroupCount);
            distributionList.add(gradesMap);
            distributionList.add(degreesMap);
            distributionList.add(subjectsMap);
            distributionList.add(groupsMap);

           /* Map<String, Long> genderCount = gpTeachers.stream()
                    .collect(Collectors.groupingBy(gpTeacher -> gpTeacher.getBasic().getGender(), Collectors.counting()));
            Map<String, Long> degreeCount = gpTeachers.stream()
                    .filter(gpTeacher -> gpTeacher.getDegree() != null)
                    .collect(Collectors.groupingBy(GpTeacher::getDegree, Collectors.counting()));*/




        }catch (Exception e) {
            throw new ServiceException(ErrorCode.SYSTEM_ERROR.getCode(), "数据转换错误");

        }
        return distributionList;
    }

    public Map<String, Object> getTeacherBase(GpTeacherDto gpTeacherDto, HttpServletRequest request) {

        Map<String, Object> gradeMap;
        try {
            gradeMap =  GroupUtil.getTeacherInfo(gpTeacherDto,new GroupUtil(env), request);
        }catch (Exception e) {
            throw new ServiceException(ErrorCode.SYSTEM_ERROR.getCode(), "数据转换错误");

        }
        return gradeMap;
    }

    private static @NotNull Map<String, Integer> getStringIntegerMap(List<School.Subject> subjects, List<SchoolTeacher> schoolTeachers) {
        Map<String, String> subjectNameMap = new HashMap<>();
        for (School.Subject subject : subjects) {
            subjectNameMap.put(subject.getId(), subject.getName());
        }
        Map<String, Integer> subjectCountMap = new HashMap<>();

        // 遍历所有教师
        for (SchoolTeacher teacher : schoolTeachers) {
            // 获取教师的科目列表
            List<String> subjectIds = teacher.getSubjectIds();
            if (subjectIds != null) {
                for (String subjectId : subjectIds) {
                    // 更新每个科目的教师数量
                    subjectCountMap.put(subjectId, subjectCountMap.getOrDefault(subjectId, 0) + 1);
                }
            }
        }
        Map<String, Integer> updatedSubjectCountMap = new HashMap<>();
        for (Map.Entry<String, Integer> entry : subjectCountMap.entrySet()) {
            String subjectId = entry.getKey();
            Integer count = entry.getValue();
            String subjectName = subjectNameMap.get(subjectId);
            if (subjectName != null) {
                updatedSubjectCountMap.put(subjectName, count);
            }
        }
        return updatedSubjectCountMap;
    }

    private static @NotNull Map<String, Integer> getPeriodCountMap(List<School.Period> periods,List<SchoolTeacher> schoolTeachers) {

        Map<String, Integer> periodCountMap = new HashMap<>();
        for (School.Period period : periods) {
            for ( School.Subject subject : period.getSubjects()) {
                for (SchoolTeacher schoolTeacher : schoolTeachers) {
                    if (schoolTeacher.getSubjectIds().contains(subject.getId())) {
                        periodCountMap.put(period.getId(), periodCountMap.getOrDefault(period.getId(), 0) + 1);
                    }
                }
            }
        }
        Map<String, String> periodNameMap = new HashMap<>();
        List<String> periodNames = new ArrayList<>();
        for (School.Period period : periods) {
            periodNameMap.put(period.getId(), period.getName());
            periodNames.add(period.getName());
        }
        //Map<String, Integer> periodCountMap = getStringIntegerMap(groupLists);
        // 更新 periodCountMap 的 key 为 period 名称
        Map<String, Integer> updatedPeriodCountMap = new HashMap<>();
        for (Map.Entry<String, Integer> entry : periodCountMap.entrySet()) {
            String periodId = entry.getKey();
            Integer count = entry.getValue();
            String periodName = periodNameMap.get(periodId);
            if (periodName != null) {
                updatedPeriodCountMap.put(periodName, count);
            }
        }
        for (String periodName : periodNames) {
            if(!updatedPeriodCountMap.containsKey(periodName)) {
                updatedPeriodCountMap.put(periodName, 0);
            }
        }
        return updatedPeriodCountMap;
    }

    /*private static @NotNull Map<String, Integer> getStringIntegerMap(List<GroupList> groupLists) {
        Map<String, Integer> periodCountMap = new HashMap<>();
        // 遍历所有 GroupList 对象
        for (GroupList group : groupLists) {
            // 获取 periodId
            String periodId = group.getPeriodId();
            if (periodId != null) {
                    // 计算成员数量
                    int memberCount = (int) group.getMembers().stream().filter(member -> member.type == 1).count();
                    // 更新 periodCountMap
                    periodCountMap.put(periodId, periodCountMap.getOrDefault(periodId, 0) + memberCount);

            }
        }
        return periodCountMap;
    }*/
}
